local ADDON_NAME, Engine = ...

--[[
	xpcall safecall implementation
	GLOBAL: geterrorhandler
]]
local xpcall = xpcall
local tconcat = table.concat

local function errorhandler(err)
	return geterrorhandler()(err)
end

local function CreateDispatcher(argCount)
	local code = [[
		local xpcall, eh = ...
		local method, ARGS
		local function call() return method(ARGS) end

		local function dispatch(func, ...)
			method = func
			if not method then return end
			ARGS = ...
			return xpcall(call, eh)
		end
	
		return dispatch
	]]

	local ARGS = {}
	for i = 1, argCount do ARGS[i] = "arg"..i end
	code = code:gsub("ARGS", tconcat(ARGS, ", "))
	return assert(loadstring(code, "safecall Dispatcher["..argCount.."]"))(xpcall, errorhandler)
end

local Dispatchers = setmetatable(
	{},
	{__index = function(self, argCount)
		local dispatcher = CreateDispatcher(argCount)
		rawset(self, argCount, dispatcher)
		return dispatcher
	end
	}
)
Dispatchers[0] = function(func)
	return xpcall(func, errorhandler)
end
 
Engine.safecall = function(func, ...)
	return Dispatchers[select("#", ...)](func, ...)
end